@@ -34,6 +34,7 @@ public class AppAddedSchedulerEvent extends SchedulerEvent {
   private final boolean isAppRecovering;
   private final Priority appPriority;
   private final ApplicationPlacementContext placementContext;
+  private boolean unmanagedAM = false;
 
   public AppAddedSchedulerEvent(ApplicationId applicationId, String queue,
       String user) {
@@ -58,6 +59,7 @@ public class AppAddedSchedulerEvent extends SchedulerEvent {
     this(submissionContext.getApplicationId(), submissionContext.getQueue(),
         user, isAppRecovering, submissionContext.getReservationID(),
         appPriority, null);
+    this.unmanagedAM = submissionContext.getUnmanagedAM();
   }
 
   public AppAddedSchedulerEvent(String user,
@@ -66,6 +68,7 @@ public class AppAddedSchedulerEvent extends SchedulerEvent {
     this(submissionContext.getApplicationId(), submissionContext.getQueue(),
         user, isAppRecovering, submissionContext.getReservationID(),
         appPriority, placementContext);
+    this.unmanagedAM = submissionContext.getUnmanagedAM();
   }
 
   public AppAddedSchedulerEvent(ApplicationId applicationId, String queue,
@@ -108,4 +111,8 @@ public class AppAddedSchedulerEvent extends SchedulerEvent {
   public ApplicationPlacementContext getPlacementContext() {
     return placementContext;
   }
+
+  public boolean isUnmanagedAM() {
+    return unmanagedAM;
+  }
 }
